package picmicrocontroller;

import gui.ConsolePane;


/**
 * This class holds all the instructions of a PIC16F877.
 * Each instruction class simulates the interior behavior of the PIC.
 */
abstract public class Instruction {
	
	protected PIC pic;
	
	protected int f;
	protected int d;
	protected int b;
	protected int k;
	protected boolean detail=false;
	protected ConsolePane console;
	/**
	 * Simulate the instruction by changing registers of PIC data memory.
	 *
	 */
	abstract public void simulate() ;
	
	public void setDetail(boolean value)
	{
		detail = value;
	}
	/**
	 * Sets the zero flag to the appropriate value according to the result
	 * @param res result of an instruction
	 */
	public void adjustZeroFlag(int res) {
		if(res == 0)
			this.pic.dataMemory.setZeroFlag();
		else
			this.pic.dataMemory.clearZeroFlag();
	}
	
	/**
	 * Sets the carry flag to the appropriate value according to the result
	 * @param res result of an instruction
	 */
	public void adjustCarry(int res) {
		if(res > 255)	//kywrd try if int=256 is correctly changed to byte?
			this.pic.dataMemory.setCarry();
		else
			this.pic.dataMemory.clearCarry();
	}
	/**
	 * Sets the digit flag to the appropriate value according to the result
	 * @param res result of an instruction
	 */
	public void adjustDigitCarry(int res) {
		if(res > 15)
			this.pic.dataMemory.setDigitCarry();
		else
			this.pic.dataMemory.clearDigitCarry();
	}
}


class I_addwf extends Instruction
{
	I_addwf(PIC pict, int ft, int dt)
	{	
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating addwf "+f+" "+d+"\n");
		
		int res = (this.pic.wreg.read()+this.pic.dataMemory.read(this.f));
		
		if(this.d == 0)
			this.pic.wreg.write((byte)res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)res);
	
		adjustZeroFlag(res);
		adjustCarry(res);
		adjustDigitCarry(res);
	}
}


class I_andwf extends Instruction
{
	I_andwf(PIC pict, int ft, int dt)
	{
		
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating andwf "+f+" "+d+"\n");	
		
		int res = (this.pic.wreg.read() & this.pic.dataMemory.read(this.f));
		
		if(this.d == 0)
			this.pic.wreg.write((byte)res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)res);
		
		adjustZeroFlag(res);
	}
}


class I_clrf extends Instruction
{
	I_clrf(PIC pict, int ft)
	{		
		this.pic = pict;
		this.f = ft;	
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating clrf "+f+"\n");	
		
		this.pic.dataMemory.clear(this.f);
	
		this.pic.dataMemory.setZeroFlag();
	}
}

class I_clrw extends Instruction
{	
	I_clrw(PIC pict)
	{
		this.pic = pict;	
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating clrw\n");
		
		this.pic.wreg.clear();
		
		this.pic.dataMemory.setZeroFlag();
	}
}

class I_comf extends Instruction
{	
	I_comf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating comf "+f+" "+d+"\n");		
		
		int res = ~(this.pic.dataMemory.read(this.f));
		
		if(this.d == 0)
			this.pic.wreg.write((byte)res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)res);
		
		adjustZeroFlag(res);
	}
}

class I_decf extends Instruction
{	
	I_decf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating decf "+f+" "+d+"\n");
		
		int res = (this.pic.dataMemory.read(this.f)-1);
		
		if(this.d == 0)
			this.pic.wreg.write((byte)res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)res);
		
		adjustZeroFlag(res);
	}
}

class I_decfsz extends Instruction
{	
	I_decfsz(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating decfsz "+f+" "+d+"\n");
	
		byte data = (byte)(this.pic.dataMemory.read(this.f)-1);
		
		if(this.d == 1)
			this.pic.dataMemory.write(this.f, data);
		else if(this.d == 0)
			this.pic.wreg.write(data);
		
		if(data == 0)
			this.pic.programCounter.increment();
	}
}

class I_incf extends Instruction
{	
	I_incf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating incf "+f+" "+d+"\n");
		
		int res = (this.pic.dataMemory.read(this.f)+1);
		
		if(this.d == 0)
			this.pic.wreg.write((byte)res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)res);
		
		adjustZeroFlag(res);
	}
}

class I_incfsz extends Instruction
{	
	I_incfsz(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating incfsz "+f+" "+d+"\n");
		
		byte data = (byte)(this.pic.dataMemory.read(this.f)+1);
		
		if(this.d == 1)
			this.pic.dataMemory.write(this.f, data);
		else if(this.d == 0)
			this.pic.wreg.write(data);
		
		if(data == 0)
			this.pic.programCounter.increment();
	}
}

class I_iorwf extends Instruction
{	
	I_iorwf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating iorwf "+f+" "+d+"\n");
		
		int res = (this.pic.wreg.read() | this.pic.dataMemory.read(this.f));
		
		if(this.d == 0)
			this.pic.wreg.write((byte)res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)res);
		
		adjustZeroFlag(res);
	}
}

class I_movf extends Instruction
{	
	I_movf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating movf "+f+" "+d+"\n");
		
		byte res = this.pic.dataMemory.read(this.f);
		
		if(this.d == 0)
			this.pic.wreg.write(res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, res);
		
		adjustZeroFlag((int)res);
	}
}

class I_movwf extends Instruction
{	
	I_movwf(PIC pict, int ft)
	{
		this.pic = pict;
		this.f = ft;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating movwf "+f+"\n");
	
		this.pic.dataMemory.write(this.f, this.pic.wreg.read());
	}
}

class I_nop extends Instruction
{	
	I_nop(PIC pict)
	{
		this.pic = pict;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating nop\n");
	}
}


class I_rlf extends Instruction
{	
	I_rlf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating rlf "+f+" "+d+"\n");
		
		int data = this.pic.dataMemory.read(this.f); 
		data = data << 1;	//kywrd (through carry? carryflag?)
		
		if(this.d == 0)
			this.pic.wreg.write((byte)data);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)data);
	}
}

class I_rrf extends Instruction
{	
	I_rrf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating rrf "+f+" "+d+"\n");
		
		int data = this.pic.dataMemory.read(this.f); 
		data = data >> 1;	//kywrd (through carry? carryflag?)
		
		if(this.d == 0)
			this.pic.wreg.write((byte)data);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)data);
	}
}

class I_subwf extends Instruction
{	
	I_subwf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating subwf "+f+" "+d+"\n");
		
		int res = (this.pic.dataMemory.read(this.f) - this.pic.wreg.read());
		
		if(this.d == 0)
			this.pic.wreg.write((byte)res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)res);
		
		adjustZeroFlag(res);
		adjustCarry(res);		//kywrd (byte or int? borrow) 
		adjustDigitCarry(res); //kywrd
	}
}

class I_swapf extends Instruction
{	
	I_swapf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating swapf "+f+" "+d+"\n");
		byte temp = this.pic.dataMemory.read(f);
		byte temp1 = (byte)(temp & Integer.parseInt("00001111"));
		byte temp2 = (byte)(temp & Integer.parseInt("11110000"));
		
		temp1 = (byte)(temp1 << 4);
		temp2 = (byte)(temp2 >> 4);
		
		byte res = (byte)(temp1 + temp2);
		
		if(this.d == 0)
			this.pic.wreg.write(res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, res);
		
		//kywrd  test?
	}
}

class I_xorf extends Instruction
{	
	I_xorf(PIC pict, int ft, int dt)
	{
		this.pic = pict;
		this.f = ft;
		this.d = dt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating xorf "+f+" "+d+"\n");
		
		int res = (this.pic.wreg.read() ^ this.pic.dataMemory.read(this.f));
		
		if(this.d == 0)
			this.pic.wreg.write((byte)res);
		else if(this.d == 1)
			this.pic.dataMemory.write(this.f, (byte)res);
		
		adjustZeroFlag(res);
	}
}

class I_bcf extends Instruction
{	
	I_bcf(PIC pict, int ft, int bt)
	{
		this.pic = pict;
		this.f = ft;
		this.b = bt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating bcf "+f+" "+b+"\n");
		
		int data =  this.pic.dataMemory.read(this.f);
		
		char[] chr = {'1', '1', '1', '1', '1', '1', '1', '1'};
		chr[7 - this.b] = '0';
		String str = new String(chr);
		
		
		data = data & Integer.parseInt(str, 2);
		
		this.pic.dataMemory.write(this.f, (byte)data);
	}
}

class I_bsf extends Instruction
{	
	I_bsf(PIC pict, int ft, int bt)
	{
		this.pic = pict;
		this.f = ft;
		this.b = bt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating bsf "+f+" "+b+"\n");
		
		int data =  this.pic.dataMemory.read(this.f);
		
		char[] chr = {'0', '0', '0', '0', '0', '0', '0', '0'};
		chr[7 - this.b] = '1';
		String str = new String(chr);
		
		data = data | Integer.parseInt(str, 2);
		
		this.pic.dataMemory.write(this.f, (byte)data);
		//System.out.println("b=" + b + "  bsf writes data=" + data);
	}
}

class I_btfsc extends Instruction
{	
	I_btfsc(PIC pict, int ft, int bt)
	{
		this.pic = pict;
		this.f = ft;
		this.b = bt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating btfsc "+f+" "+b+"\n");
		
		String str = Integer.toBinaryString(this.pic.dataMemory.read(this.f));
		//System.out.println(this.pic.dataMemory.read(this.f) + " " + b);
		
		char[] chr = str.toCharArray();
		char[] ch2 = new char[8];
		int i, j;
		for(i=0; i<8; i++) {
			ch2[i] = 0;
		}
		for(i=7, j = str.length()-1; j>=0; i--, j--) {
			ch2[i] = chr[j];
			//System.out.println(j + " " + chr[j]);
		}
		
		if(ch2[7 - this.b] == 0)
			this.pic.programCounter.increment();
	}
}

class I_btfss extends Instruction	
{	
	I_btfss(PIC pict, int ft, int bt)
	{
		this.pic = pict;
		this.f = ft;
		this.b = bt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating btfss "+f+" "+b+"\n");

		int itemp = (int) Math.pow(2, this.b);
	
		itemp = itemp & this.pic.dataMemory.read(this.f);
	
		if(itemp != 0)
			this.pic.programCounter.increment();
			

	}
}

class I_addlw extends Instruction
{	
	I_addlw(PIC pict, int kt)
	{
		this.pic = pict;
		this.k = kt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating addlw "+k+"\n");
		
		int res = (this.pic.wreg.read() + this.k);
		
		this.pic.wreg.write((byte)res);
		
		adjustZeroFlag(res);
		adjustCarry(res);
		adjustDigitCarry(res);
	}
}

class I_andlw extends Instruction
{	
	I_andlw(PIC pict, int kt)
	{
		this.pic = pict;
		this.k = kt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating andlw "+k+"\n");
		
		int res = (this.pic.wreg.read() & this.k);
		
		this.pic.wreg.write((byte)res);
		
		adjustZeroFlag(res);
	}
}

class I_call extends Instruction
{	
	I_call(PIC pict, int kt)
	{
		this.pic = pict;
		this.k = kt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating call "+k+"\n");
		
		//byte twoPCLATHbits = this.pic.dataMemory.read(10);
		this.pic.ProgramCounterStack.push(this.pic.programCounter.getPC());
		this.pic.programCounter.setPC(k);
		
	}
}

class I_clrwdt extends Instruction
{	
	I_clrwdt(PIC pict)
	{
		this.pic = pict;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating clrwdt\n");
		this.pic.WDT.clear();
		
		byte stat = this.pic.dataMemory.read(3);		//adjust status to bank 2, to be able to reach OPTION_REG
		byte stat2 = (byte)(stat & Byte.parseByte("10111111", 2));
		stat2 = (byte)(stat2 | Byte.parseByte("00100000", 2));
		this.pic.dataMemory.write(3, stat2);
		
		byte temp = this.pic.dataMemory.read(1);		//read OPTION_REG to adjust WDT prescaler
		temp = (byte)(temp & Byte.parseByte("11111000", 2));
		temp = (byte)(temp | Byte.parseByte("00001000", 2));
		this.pic.dataMemory.write(1, temp);
		
		stat = (byte)(stat | Byte.parseByte("00011000", 2));
		this.pic.dataMemory.write(3, stat);		//restore the bank, and set TO and PD bits
		//kywrd  check if bits set?
	}
}


class I_goto extends Instruction
{	
	I_goto(PIC pict, int kt)
	{
		this.pic = pict;
		this.k = kt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating goto " + k+"\n");
		
		this.pic.programCounter.setPC(this.k);  //kywrd pclath 3 4?
	}
}

class I_iorlw extends Instruction
{	
	I_iorlw(PIC pict, int kt)
	{
		this.pic = pict;
		this.k = kt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating iorlw "+k+"\n");
		
		int res = (this.pic.wreg.read() | this.k);
		
		this.pic.wreg.write((byte)res);
		
		adjustZeroFlag(res);
	}
}

class I_movlw extends Instruction
{	
	I_movlw(PIC pict, int kt)
	{
		this.pic = pict;
		this.k = kt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating movlw "+k+"\n");
		
		this.pic.wreg.write((byte)this.k);
	}
}

class I_retfie extends Instruction
{	
	I_retfie(PIC pict)
	{
		this.pic = pict;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating retfie\n");
		this.pic.programCounter.setPC(((Integer)this.pic.ProgramCounterStack.pop()).intValue());   //kywrd ?
		byte intcon = this.pic.dataMemory.read(11);
		intcon = (byte)(intcon | Integer.parseInt("10000000"));
		this.pic.dataMemory.write(11, intcon);
	}
}



class I_retlw extends Instruction
{	
	I_retlw(PIC pict, int kt)
	{
		this.pic = pict;
		this.k = kt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating retlw "+k+"\n");
		this.pic.programCounter.setPC(((Integer)this.pic.ProgramCounterStack.pop()).intValue());   //kywrd ?
		this.pic.wreg.write((byte)k);
		//kywrd
	}
}

class I_return extends Instruction
{	
	I_return(PIC pict)
	{
		this.pic = pict;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating return\n");
		this.pic.programCounter.setPC(((Integer)this.pic.ProgramCounterStack.pop()).intValue());   //kywrd ?
		//kywrd
	}
}


class I_sleep extends Instruction
{	
	I_sleep(PIC pict)
	{
		this.pic = pict;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating sleep\n");
		
		this.pic.WDT.clear();
		
		byte stat = this.pic.dataMemory.read(3);		//adjust status to bank 2, to be able to reach OPTION_REG
		byte stat2 = (byte)(stat & Byte.parseByte("10111111", 2));
		stat2 = (byte)(stat2 | Byte.parseByte("00100000", 2));
		this.pic.dataMemory.write(3, stat2);
		
		byte temp = this.pic.dataMemory.read(1);		//read OPTION_REG to adjust WDT prescaler
		temp = (byte)(temp & Byte.parseByte("11111000", 2));
		temp = (byte)(temp | Byte.parseByte("00001000", 2));
		this.pic.dataMemory.write(1, temp);
		
		stat = (byte)(stat | Byte.parseByte("00010000", 2));
		stat = (byte)(stat & Byte.parseByte("11110111", 2));
		this.pic.dataMemory.write(3, stat);		//restore the bank, set TO, clear PD bits in STATUS
	
		this.pic.sleep();
		//kywrd ok?
	}
}

class I_sublw extends Instruction
{	
	I_sublw(PIC pict, int kt)
	{
		this.pic = pict;
		this.k = kt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating sublw "+k+"\n");
		
		int res = (this.k - this.pic.wreg.read());
		
		this.pic.wreg.write((byte)res);
		
		adjustZeroFlag(res);	//kywrd borrow? int byte
		adjustCarry(res);
		adjustDigitCarry(res);
	}
}



class I_xorlw extends Instruction
{	
	I_xorlw(PIC pict, int kt)
	{
		this.pic = pict;
		this.k = kt;
		this.console = pic.getSimulatorFrame().consolePane;
		//System.out.println("addwf created f=" + f + " d=" + d );
	}
	
	@Override
	public void simulate()
	{
		if(detail)
			console.addText("simulating xorlw "+k+"\n");
		
		int res = (this.pic.wreg.read() ^ this.k);
		
		this.pic.wreg.write((byte)res);
		
		adjustZeroFlag(res);
	}
}
